@@ -1211,7 +1211,11 @@ void identify (__u16 *id_supplied)
 		__u16 word69 = val[69] & ~(trimz | trimd); /* TRIM bits require special interpretation */
 		print_features(word69, word69, feat_word69_str);
 		if (val[169] & 1 && val[169] != 0xffff) { /* supports TRIM ? */
-			printf("\t   *\tData Set Management TRIM supported\n");
+			printf("\t   *\tData Set Management TRIM supported");
+			if (val[105] && val[105] != 0xffff)
+				printf(" (limit %u block%s)\n", val[105], val[105] > 1 ? "s" : "");
+			else
+				printf(" (limit unknown)\n");
 			if (val[69] & trimd) { /* Deterministic TRIM support */
 				if (val[69] & trimz)
 					print_features(trimz, trimz, feat_word69_str);
@@ -1322,12 +1326,12 @@ void identify (__u16 *id_supplied)
 				else				  printf("high\n");
 			}
 		}
-		jj =  val[ERASE_TIME]     & ERASE_BITS;
-		kk =  val[ENH_ERASE_TIME] & ERASE_BITS;
-		if(jj || kk) {
+		jj =  val[ERASE_TIME];
+		kk =  val[ENH_ERASE_TIME];
+		if((jj && jj <= 0x00ff) || (kk && kk <= 0x00ff)) {
 			printf("\t");
-			if(jj) printf("%umin for SECURITY ERASE UNIT. ", jj==ERASE_BITS ? 508 : jj<<1);
-			if(kk) printf("%umin for ENHANCED SECURITY ERASE UNIT.", kk==ERASE_BITS ? 508 : kk<<1);
+			if(jj) printf("%umin for SECURITY ERASE UNIT. ",         (jj == 0xff) ? 508 : (jj * 2));
+			if(kk) printf("%umin for ENHANCED SECURITY ERASE UNIT.", (kk == 0xff) ? 508 : (kk * 2));
 			printf("\n");
 		}
 	}
