@@ -55,6 +55,30 @@ static const int default_timeout_secs = 15;
  *	SG_DXFER_TO_DEV, SG_DXFER_FROM_DEV, SG_DXFER_NONE
  */
 
+#if 0  /* maybe use this in sg16 later.. ? */
+static inline int get_rw (__u8 ata_op)
+{
+	switch (ata_op) {
+		case ATA_OP_DSM:
+		case ATA_OP_WRITE_PIO:
+		case ATA_OP_WRITE_LONG:
+		case ATA_OP_WRITE_LONG_ONCE:
+		case ATA_OP_WRITE_PIO_EXT:
+		case ATA_OP_WRITE_DMA_EXT:
+		case ATA_OP_WRITE_FPDMA:
+		case ATA_OP_WRITE_UNC_EXT:
+		case ATA_OP_WRITE_DMA:
+		case ATA_OP_SECURITY_UNLOCK:
+		case ATA_OP_SECURITY_DISABLE:
+		case ATA_OP_SECURITY_ERASE_UNIT:
+		case ATA_OP_SECURITY_SET_PASS:
+			return SG_WRITE;
+		default:
+			return SG_READ;
+	}
+}
+#endif
+
 static inline int needs_lba48 (__u8 ata_op, __u64 lba, unsigned int nsect)
 {
 	switch (ata_op) {
@@ -167,7 +191,7 @@ int sg16 (int fd, int rw, int dma, struct ata_tf *tf,
 	unsigned char cdb[SG_ATA_16_LEN];
 	unsigned char sb[32], *desc;
 	struct scsi_sg_io_hdr io_hdr;
-	int prefer12 = prefer_ata12;
+	int prefer12 = prefer_ata12, demanded_sense = 0;
 
 	if (tf->command == ATA_OP_PIDENTIFY)
 		prefer12 = 0;
@@ -184,10 +208,13 @@ int sg16 (int fd, int rw, int dma, struct ata_tf *tf,
 	} else {
 		cdb[1] = data ? (rw ? SG_ATA_PROTO_PIO_OUT : SG_ATA_PROTO_PIO_IN) : SG_ATA_PROTO_NON_DATA;
 	}
-	cdb[ 2] = SG_CDB2_CHECK_COND;
+
+	/* libata/AHCI workaround: don't demand sense data for IDENTIFY commands */
 	if (data) {
 		cdb[2] |= SG_CDB2_TLEN_NSECT | SG_CDB2_TLEN_SECTORS;
 		cdb[2] |= rw ? SG_CDB2_TDIR_TO_DEV : SG_CDB2_TDIR_FROM_DEV;
+	} else {
+		cdb[2] = SG_CDB2_CHECK_COND;
 	}
 
 	if (!prefer12 || tf->is_lba48) {
@@ -277,7 +304,7 @@ int sg16 (int fd, int rw, int dma, struct ata_tf *tf,
 			static int second_try = 0;
 			if (!second_try++)
 				fprintf(stderr, "SG_IO: questionable sense data, results may be incorrect\n");
-		} else {
+		} else if (demanded_sense) {
 			static int second_try = 0;
 			if (!second_try++)
 				fprintf(stderr, "SG_IO: missing sense data, results may be incorrect\n");
@@ -344,13 +371,20 @@ int do_drive_cmd (int fd, unsigned char *args, unsigned int timeout_secs)
 		goto use_legacy_ioctl;
 	/*
 	 * Reformat and try to issue via SG_IO:
+	 * args[0]: command in; status out.
+	 * args[1]: lbal for SMART, nsect for all others; error out
+	 * args[2]: feat in; nsect out.
+	 * args[3]: data-count (512 multiple) for all cmds.
 	 */
-	if (args[3] && args[0] != ATA_OP_SETFEATURES) {
-		data_bytes = args[3] * 512;
-		data       = args + 4;
+	tf_init(&tf, args[0], 0, 0);
+	tf.lob.nsect = args[1];
+	tf.lob.feat  = args[2];
+	if (args[3]) {
+		data_bytes   = args[3] * 512;
+		data         = args + 4;
+		if (!tf.lob.nsect)
+			tf.lob.nsect = args[3];
 	}
-	tf_init(&tf, args[0], 0, args[1]);
-	tf.lob.feat = args[2];
 	if (tf.command == ATA_OP_SMART) {
 		tf.lob.nsect = args[3];
 		tf.lob.lbal  = args[1];
