@@ -3,6 +3,7 @@
 #include <linux/types.h>
 
 enum {
+	ATA_OP_DSM			= 0x06, // Data Set Management (TRIM)
 	ATA_OP_READ_PIO			= 0x20,
 	ATA_OP_READ_PIO_ONCE		= 0x21,
 	ATA_OP_READ_LONG		= 0x22,
@@ -115,30 +116,30 @@ enum {
 
 struct reg_flags {
 	union {
-	unsigned lob_all		: 8;
-	struct {
-		unsigned data		: 1;
-		unsigned feat		: 1;
-		unsigned lbal		: 1;
-		unsigned nsect		: 1;
-		unsigned lbam		: 1;
-		unsigned lbah		: 1;
-		unsigned dev		: 1;
-		unsigned command	: 1;
-	} lob;
+		unsigned lob_all		: 8;
+		struct {
+			unsigned data		: 1;
+			unsigned feat		: 1;
+			unsigned lbal		: 1;
+			unsigned nsect		: 1;
+			unsigned lbam		: 1;
+			unsigned lbah		: 1;
+			unsigned dev		: 1;
+			unsigned command	: 1;
+		} lob;
 	};
 	union {
-	unsigned hob_all		: 8;
-	struct {
-		unsigned data		: 1;
-		unsigned feat		: 1;
-		unsigned lbal		: 1;
-		unsigned nsect		: 1;
-		unsigned lbam		: 1;
-		unsigned lbah		: 1;
-		unsigned dev		: 1;
-		unsigned command	: 1;
-	} hob;
+		unsigned hob_all		: 8;
+		struct {
+			unsigned data		: 1;
+			unsigned feat		: 1;
+			unsigned lbal		: 1;
+			unsigned nsect		: 1;
+			unsigned lbam		: 1;
+			unsigned lbah		: 1;
+			unsigned dev		: 1;
+			unsigned command	: 1;
+		} hob;
 	};
 };
 
@@ -223,7 +224,7 @@ struct scsi_sg_io_hdr {
 void tf_init (struct ata_tf *tf, __u8 ata_op, __u64 lba, unsigned int nsect);
 __u64 tf_to_lba (struct ata_tf *tf);
 int sg16 (int fd, int rw, int dma, struct ata_tf *tf, void *data, unsigned int data_bytes, unsigned int timeout_secs);
-int do_drive_cmd (int fd, unsigned char *args);
+int do_drive_cmd (int fd, unsigned char *args, unsigned int timeout);
 int do_taskfile_cmd (int fd, struct hdio_taskfile *r, unsigned int timeout_secs);
 int dev_has_sgio (int fd);
 void init_hdio_taskfile (struct hdio_taskfile *r, __u8 ata_op, int rw, int force_lba48,
